package hu.bme.mit.socialnetwork.serializer;

import com.google.inject.Inject;
import com.google.inject.Provider;
import hu.bme.mit.socialnetwork.services.SocialNetworkLanguageGrammarAccess;
import hu.bme.mit.socialnetwork.socialNetworkLanguage.Acquaintance;
import hu.bme.mit.socialnetwork.socialNetworkLanguage.Community;
import hu.bme.mit.socialnetwork.socialNetworkLanguage.Person;
import hu.bme.mit.socialnetwork.socialNetworkLanguage.SocialNetwork;
import hu.bme.mit.socialnetwork.socialNetworkLanguage.SocialNetworkLanguagePackage;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.serializer.acceptor.ISemanticSequenceAcceptor;
import org.eclipse.xtext.serializer.acceptor.SequenceFeeder;
import org.eclipse.xtext.serializer.diagnostic.ISemanticSequencerDiagnosticProvider;
import org.eclipse.xtext.serializer.diagnostic.ISerializationDiagnostic.Acceptor;
import org.eclipse.xtext.serializer.sequencer.AbstractSemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.GenericSequencer;
import org.eclipse.xtext.serializer.sequencer.ISemanticNodeProvider.INodesForEObjectProvider;
import org.eclipse.xtext.serializer.sequencer.ISemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService.ValueTransient;

@SuppressWarnings("restriction")
public class AbstractSocialNetworkLanguageSemanticSequencer extends AbstractSemanticSequencer {

	@Inject
	protected SocialNetworkLanguageGrammarAccess grammarAccess;
	
	@Inject
	protected ISemanticSequencerDiagnosticProvider diagnosticProvider;
	
	@Inject
	protected ITransientValueService transientValues;
	
	@Inject
	@GenericSequencer
	protected Provider<ISemanticSequencer> genericSequencerProvider;
	
	protected ISemanticSequencer genericSequencer;
	
	
	@Override
	public void init(ISemanticSequencer sequencer, ISemanticSequenceAcceptor sequenceAcceptor, Acceptor errorAcceptor) {
		super.init(sequencer, sequenceAcceptor, errorAcceptor);
		this.genericSequencer = genericSequencerProvider.get();
		this.genericSequencer.init(sequencer, sequenceAcceptor, errorAcceptor);
	}
	
	public void createSequence(EObject context, EObject semanticObject) {
		if(semanticObject.eClass().getEPackage() == SocialNetworkLanguagePackage.eINSTANCE) switch(semanticObject.eClass().getClassifierID()) {
			case SocialNetworkLanguagePackage.ACQUAINTANCE:
				if(context == grammarAccess.getAcquaintanceRule()) {
					sequence_Acquaintance_Acquaintance(context, (Acquaintance) semanticObject); 
					return; 
				}
				else break;
			case SocialNetworkLanguagePackage.COMMUNITY:
				if(context == grammarAccess.getSocialEntityRule() ||
				   context == grammarAccess.getCommunityRule()) {
					sequence_Community_Community(context, (Community) semanticObject); 
					return; 
				}
				else break;
			case SocialNetworkLanguagePackage.PERSON:
				if(context == grammarAccess.getSocialEntityRule() ||
				   context == grammarAccess.getPersonRule()) {
					sequence_Person_Person(context, (Person) semanticObject); 
					return; 
				}
				else break;
			case SocialNetworkLanguagePackage.SOCIAL_NETWORK:
				if(context == grammarAccess.getSocialNetworkRule()) {
					sequence_SocialNetwork_SocialNetwork(context, (SocialNetwork) semanticObject); 
					return; 
				}
				else break;
			}
		if (errorAcceptor != null) errorAcceptor.accept(diagnosticProvider.createInvalidContextOrTypeDiagnostic(semanticObject, context));
	}
	
	/**
	 * Constraint:
	 *     (source=[Person|ID] type=AcquaintanceType target=[Person|ID])
	 *
	 * Features:
	 *    source[1, 1]
	 *    type[1, 1]
	 *    target[1, 1]
	 */
	protected void sequence_Acquaintance_Acquaintance(EObject context, Acquaintance semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__SOURCE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__SOURCE));
			if(transientValues.isValueTransient(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__TYPE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__TYPE));
			if(transientValues.isValueTransient(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__TARGET) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__TARGET));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getAcquaintanceAccess().getSourcePersonIDTerminalRuleCall_1_0_1(), semanticObject.getSource());
		feeder.accept(grammarAccess.getAcquaintanceAccess().getTypeAcquaintanceTypeEnumRuleCall_3_0(), semanticObject.getType());
		feeder.accept(grammarAccess.getAcquaintanceAccess().getTargetPersonIDTerminalRuleCall_4_0_1(), semanticObject.getTarget());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (name=ID (children+=Community children+=Community*)?)
	 *
	 * Features:
	 *    name[1, 1]
	 *    children[0, *]
	 */
	protected void sequence_Community_Community(EObject context, Community semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (name=ID sex=Sex (membership+=[Community|ID] membership+=[Community|ID]*)?)
	 *
	 * Features:
	 *    name[1, 1]
	 *    sex[1, 1]
	 *    membership[0, *]
	 */
	protected void sequence_Person_Person(EObject context, Person semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (entities+=SocialEntity+ acquaintances+=Acquaintance*)
	 *
	 * Features:
	 *    entities[1, *]
	 *    acquaintances[0, *]
	 */
	protected void sequence_SocialNetwork_SocialNetwork(EObject context, SocialNetwork semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
}
